package painter;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

@SuppressWarnings("serial")
public class Canvas extends JPanel {

	private static final int CANVAS_HEIGHT = 400;

	private static final int CANVAS_WIDTH = 400;

	private Color strokeColor;

	private Color backgroundColor;

	private int strokeSize;

	private Dimension dimension = new Dimension(CANVAS_WIDTH, CANVAS_HEIGHT);

	private Image image;

	private Point lastPos;

	public Canvas(Color strokeColor, Color backgroundColor, int strokeSize) {
		this.strokeColor = strokeColor;
		this.backgroundColor = backgroundColor;
		this.strokeSize = strokeSize;

		reInit(false);
		setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
		setPreferredSize(dimension);

		addMouseListener(new MouseAdapter() {
			public void mousePressed(final MouseEvent e) {
				Point currPos = e.getPoint();
				drawLine(currPos, currPos);
				repaint();
				lastPos = currPos;
			}
		});

		addMouseMotionListener(new MouseMotionAdapter() {

			@Override
			public void mouseDragged(MouseEvent e) {
				Point currPos = e.getPoint();
				drawLine(currPos, lastPos);
				repaint();
				lastPos = currPos;
			}
		});

		addComponentListener(new ComponentAdapter() {

			@Override
			public void componentResized(ComponentEvent e) {
				Dimension d = e.getComponent().getSize();
				if (d.width > dimension.width || d.height > dimension.height) {
					dimension = d;
					reInit(true);
					repaint();
				}
			}
		});
	}

	public void setStrokeSize(int strokeSize) {
		this.strokeSize = strokeSize;
	}

	public void setStrokeColor(Color strokeColor) {
		this.strokeColor = strokeColor;
	}

	public Color getStrokeColor() {
		return this.strokeColor;
	}

	public void clear() {
		reInit(false);
		repaint();
	}

	@Override
	public void paintComponent(Graphics g) {
		g.drawImage(image, 0, 0, this);
	}

	private void reInit(boolean resize) {
		Image old = image;
		image = new BufferedImage(dimension.width, dimension.height,
				BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = (Graphics2D) image.getGraphics();
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setBackground(backgroundColor);
		g2d.clearRect(0, 0, dimension.width, dimension.height);

		if (resize) {
			g2d.drawImage(old, 0, 0, this);
		}
	}

	private void drawLine(Point from, Point to) {
		Graphics2D g2d = (Graphics2D) image.getGraphics();
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		g2d.setColor(strokeColor);
		g2d.setStroke(new BasicStroke(strokeSize, BasicStroke.CAP_ROUND,
				BasicStroke.JOIN_MITER));
		g2d.drawLine(from.x, from.y, to.x, to.y);
	}
}